@@ -19,9 +19,9 @@
  */
 
 #include "field.h"
-#include <glom/libglom/connectionpool.h>
+#include <libglom/connectionpool.h>
 #include "glomconversions.h"
-#include <glom/libglom/utils.h>
+#include <libglom/utils.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -253,8 +253,23 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnom
 
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
-  //TODO: Handle exceptions as in BaseDB::connect().
-  sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect();
+  sharedptr<SharedConnection> connection;
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try
+  {
+    connection = ConnectionPool::get_instance()->connect();
+  }
+  catch (Glom::ExceptionConnection& ex)
+  {
+    //TODO: Do something more useful here
+    std::cerr << "Field::sql, connection failed: " << ex.what() << std::endl;
+  }
+#else
+  std::auto_ptr<Glom::ExceptionConnection> ex;
+  connection = ConnectionPool::get_instance()->connect(ex);
+  if (ex.get())
+    std::cerr << "Field::sql, connection failed: " << ex->what() << std::endl;
+#endif
   if(connection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connection->get_gda_connection();
@@ -265,6 +280,8 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
   return Glib::ustring();
 }
 
+#define GLOM_QUOTE_FOR_FILE_FORMAT "\""
+
 Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value) const
 {
   return to_file_format(value, m_glom_type);
@@ -289,7 +306,10 @@ Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value, glom_field_t
   }
   
   NumericFormat format_ignored; //Because we use ISO format.
-  return Conversions::get_text_for_gda_value(glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
+  const Glib::ustring result = Conversions::get_text_for_gda_value(glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
+  
+  //Escape " as "", as specified by the CSV RFC:
+  return Utils::string_replace(result, GLOM_QUOTE_FOR_FILE_FORMAT, GLOM_QUOTE_FOR_FILE_FORMAT GLOM_QUOTE_FOR_FILE_FORMAT);
 }
 
 namespace
@@ -316,8 +336,11 @@ Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, glom_field_t
   
   if(glom_type == TYPE_IMAGE)
   {
+    if(str.empty())
+      return  Gnome::Gda::Value();
+
     GdaBinary* gdabinary = gda_string_to_binary(str.c_str());
-    if(!success)
+    if(!success || !gdabinary)
       return Gnome::Gda::Value();
     else
     {
@@ -519,7 +542,12 @@ Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& valu
   */
 
   Glib::RefPtr<Gnome::Gda::Holder> holder = Gnome::Gda::Holder::create(gtype, real_name);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   holder->set_value_as_value(value);
+#else
+  std::auto_ptr<Glib::Error> ex;
+  holder->set_value_as_value(value, ex);
+#endif
   return holder;
 }
 
