@@ -18,12 +18,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "box_data_calendar_related.h"
-#include "dialog_layout_calendar_related.h"
+#include <glom/mode_data/box_data_calendar_related.h>
+#include <glom/mode_design/layout/dialog_layout_calendar_related.h>
+#include <glom/utils_ui.h>
 #include <glom/application.h>
-#include <glom/libglom/data_structure/glomconversions.h>
+#include <libglom/data_structure/glomconversions.h>
 #include <glom/frame_glom.h> //For show_ok_dialog()
-#include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -65,18 +65,38 @@ void Box_Data_Calendar_Related::enable_buttons()
   //m_calendar.set_allow_view_details(view_details_possible); //Don't allow the user to go to a record in a hidden table.
 }
 
-bool Box_Data_Calendar_Related::init_db_details(const sharedptr<const LayoutItem_CalendarPortal>& portal, bool show_title)
+bool Box_Data_Calendar_Related::init_db_details(const sharedptr<const LayoutItem_Portal>& portal, bool show_title)
 {
-  m_portal = glom_sharedptr_clone(portal);
+  //This calls the other method overload:
+  return Box_Data_Portal::init_db_details(portal, show_title);
+}
 
-  LayoutWidgetBase::m_table_name = m_portal->get_table_used(Glib::ustring() /* parent table_name, not used. */); 
- Base_DB_Table::m_table_name = LayoutWidgetBase::m_table_name;
+bool Box_Data_Calendar_Related::init_db_details(const Glib::ustring& parent_table, bool show_title)
+{
+  //std::cout << "DEBUG: Box_Data_Calendar_Related::init_db_details(): " << parent_table << std::endl;
 
-  const Glib::ustring relationship_title = m_portal->get_title_used(Glib::ustring() /* parent title - not relevant */);
-  
+  m_parent_table = parent_table;
+
+  if(m_portal)
+    LayoutWidgetBase::m_table_name = m_portal->get_table_used(Glib::ustring() /* parent table_name, not used. */); 
+  else
+    LayoutWidgetBase::m_table_name = Glib::ustring();
+
+  Base_DB_Table::m_table_name = LayoutWidgetBase::m_table_name;
+
+  //TODO: This is duplicated in box_data_related_list.cc and box_data_portal.cc. Just use code from the base class?
   if(show_title)
   {
-    m_Label.set_markup(Bakery::App_Gtk::util_bold_message(relationship_title));
+    Glib::ustring relationship_title;
+    if(m_portal && m_portal->get_has_relationship_name())
+      relationship_title = m_portal->get_title_used(Glib::ustring() /* parent title - not relevant */);
+    else
+    {
+      //Note to translators: This text is shown instead of a table title, when the table has not yet been chosen.
+      relationship_title = _("Undefined Table");
+    }
+
+    m_Label.set_markup(Utils::bold_message(relationship_title));
     m_Label.show();
 
     m_Alignment.set_padding(Utils::DEFAULT_SPACING_SMALL /* top */, 0, Utils::DEFAULT_SPACING_LARGE /* left */, 0);
@@ -89,13 +109,16 @@ bool Box_Data_Calendar_Related::init_db_details(const sharedptr<const LayoutItem
     m_Alignment.set_padding(0, 0, 0, 0); //The box itself has padding of 6.
   }
 
-  m_key_field = get_fields_for_table_one_field(LayoutWidgetBase::m_table_name, m_portal->get_to_field_used());
+  if(m_portal)
+    m_key_field = get_fields_for_table_one_field(LayoutWidgetBase::m_table_name, m_portal->get_to_field_used());
+  else
+    m_key_field.clear();
 
   enable_buttons();
 
   FoundSet found_set;
   found_set.m_table_name = LayoutWidgetBase::m_table_name;
-  return Box_Data::init_db_details(found_set); //Calls create_layout() and fill_from_database().
+  return Box_Data::init_db_details(found_set, "" /* layout_platform */); //Calls create_layout() and fill_from_database().
 }
 
 bool Box_Data_Calendar_Related::fill_from_database()
@@ -146,6 +169,7 @@ bool Box_Data_Calendar_Related::fill_from_database()
 
     //Add an AND to the existing where clause, to get only records within these dates, if any:
     sharedptr<const Field> date_field = derived_portal->get_date_field();
+    //TODO: Use a SQL parameter instead of using sql().
     const Glib::ustring extra_where_clause = "\"" + where_clause_to_table_name + "\".\"" + date_field->get_name() + "\""
       " BETWEEN DATE " + date_field->sql(date_start_value) + 
       " AND DATE " + date_field->sql(date_end_value);
@@ -162,7 +186,7 @@ bool Box_Data_Calendar_Related::fill_from_database()
     
     const Glib::ustring sql_query = Utils::build_sql_select_with_where_clause(m_found_set.m_table_name, m_FieldsShown, where_clause, m_found_set.m_extra_join, m_found_set.m_sort_clause, m_found_set.m_extra_group_by);
     //std::cout << "DEBUG: sql_query=" << sql_query << std::endl;
-    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, get_app_window());
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute_select(sql_query);
     if(!(datamodel))
       return true;
     
@@ -178,14 +202,26 @@ bool Box_Data_Calendar_Related::fill_from_database()
        continue;
        
       //Get the date value for this row:
-      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);     
+#ifdef GLIBMM_EXCEPTIONS_ENABLED      
+      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index);
+#else
+      std::auto_ptr<Glib::Error> error;      
+      Gnome::Gda::Value value_date = datamodel->get_value_at(m_query_column_date_field, row_index, error);
+#endif      
       const Glib::Date date = value_date.get_date();
       
       //Get all the values for this row:
       type_vector_values* pVector = new type_vector_values(m_FieldsShown.size());
       for(int column_index = 0; column_index < columns_count; ++column_index)
       {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED 
         (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index);
+#else
+        std::auto_ptr<Glib::Error> error;      
+          (*pVector)[column_index] = datamodel->get_value_at(column_index, row_index, error);
+        if (error.get())
+          break;
+#endif 
       }
       
       m_map_values[date].push_back(pVector);
@@ -248,12 +284,15 @@ void Box_Data_Calendar_Related::on_record_added(const Gnome::Gda::Value& primary
     sharedptr<Field> field_primary_key; //TODO: = m_calendar.get_key_field();
 
     //Create the link by setting the foreign key
-    if(m_key_field)
+    if(m_key_field && m_portal)
     {
+      Glib::RefPtr<Gnome::Gda::Set> params = Gnome::Gda::Set::create();
+      params->add_holder(m_key_field->get_holder(m_key_value));
+      params->add_holder(field_primary_key->get_holder(primary_key_value));
       Glib::ustring strQuery = "UPDATE \"" + m_portal->get_table_used(Glib::ustring() /* not relevant */) + "\"";
-      strQuery += " SET \"" +  /* get_table_name() + "." +*/ m_key_field->get_name() + "\" = " + m_key_field->sql(m_key_value);
-      strQuery += " WHERE \"" + get_table_name() + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->sql(primary_key_value);
-      const bool test = query_execute(strQuery, get_app_window());
+      strQuery += " SET \"" +  /* get_table_name() + "." +*/ m_key_field->get_name() + "\" = " + m_key_field->get_gda_holder_string();
+      strQuery += " WHERE \"" + get_table_name() + "\".\"" + field_primary_key->get_name() + "\" = " + field_primary_key->get_gda_holder_string();
+      const bool test = query_execute(strQuery, params);
       if(test)
       {
         //Show it on the view, if it's visible:
@@ -314,17 +353,18 @@ void Box_Data_Calendar_Related::on_dialog_layout_hide()
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 Dialog_Layout* Box_Data_Calendar_Related::create_layout_dialog() const
 {
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom_developer.glade", "window_data_layout");
+  Glib::RefPtr<Gtk::Builder> refXml = Gtk::Builder::create_from_file(
+      GLOM_PKGDATADIR G_DIR_SEPARATOR_S "glade" G_DIR_SEPARATOR_S "glom_developer.glade",
+      "window_data_layout");
   if(refXml)
   {
     Dialog_Layout_Calendar_Related* dialog = 0;
     refXml->get_widget_derived("window_data_layout", dialog);
     return dialog;
   }
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   return NULL;
 }
@@ -335,8 +375,16 @@ void Box_Data_Calendar_Related::prepare_layout_dialog(Dialog_Layout* dialog)
   g_assert(related_dialog != NULL);
   
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
-  related_dialog->set_document(m_layout_name, get_document(), derived_portal);
+  if(derived_portal && derived_portal->get_has_relationship_name())
+  {
+    related_dialog->set_document(m_layout_name, m_layout_platform, get_document(), derived_portal);
+  }
+  else
+  {
+    related_dialog->set_document(m_layout_name, m_layout_platform, get_document(), m_parent_table);
+  }
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void Box_Data_Calendar_Related::on_calendar_month_changed()
 {
@@ -349,7 +397,7 @@ Glib::ustring Box_Data_Calendar_Related::on_calendar_details(guint year, guint m
   sharedptr<LayoutItem_CalendarPortal> derived_portal = sharedptr<LayoutItem_CalendarPortal>::cast_dynamic(m_portal);
   if(!derived_portal)
   {
-    std::cout << "DEBUG: Box_Data_Calendar_Related::on_calendar_details(): date_field is NULL" << std::endl;
+    //std::cout << "DEBUG: Box_Data_Calendar_Related::on_calendar_details(): date_field is NULL" << std::endl;
     return Glib::ustring();
   }
   
@@ -563,7 +611,7 @@ void Box_Data_Calendar_Related::on_MenuPopup_activate_layout()
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
-Gnome::Gda::Value Box_Data_Calendar_Related::get_primary_key_value(const Gtk::TreeModel::iterator& row) const
+Gnome::Gda::Value Box_Data_Calendar_Related::get_primary_key_value(const Gtk::TreeModel::iterator& /* row */) const
 {
   return Gnome::Gda::Value(); //TODO: m_AddDel.get_value_key(row);
 }
